#include <unistd.h>
#include <errno.h>
#include "socket.h"
#include "stock.h"

/* read from a socket.
 * read in a loop because fewer bytes might be read because of sockets. 
 * if read returns 0, it indicates that the socket on other side is closed. */

void sock_read (int socket, void *buf, int n) {
   register int bcount=0, br=0;
   
   while (bcount < n) {
      br = read (socket, buf, n-bcount);
      if (br > 0) {
	 bcount += br;
	 buf += br;
      }
      else if (errno == EINTR)
	continue;
      else if (br <= 0) {
	 close (socket);
#ifdef DEBUG
	 perror ("sock_read - network error");
#endif
	 exit (1);
      }
   }
}

/* write to a socket.
 * write in a loop to make sure that all the stuff is written. 
 * if EPIPE is set, it indicates that write is performed on close socket. */

void sock_write (int socket, void *buf, int n) {
   register int bcount=0, bw=0;
   
   while (bcount < n) {
      bw = write (socket, buf, n-bcount);
      if (bw >= 0) {
	 bcount += bw;
	 buf += bw;
      }
      else if (errno == EINTR) 
	continue;
      else if (bw < 0) {
	 close (socket);
#ifdef DEBUG
	 perror ("sock_write - network error");
#endif
	 exit (0);
      }
   }
}

char *sock_read_string (int socket) {
   char c;
   register int i=0;
   int resize = 4;   
   char *buf = NULL;
   
   do {
      if ((i % resize) == 0) 
	buf = (char *) realloc (buf, (i + resize) * sizeof (char));
      sock_read (socket, &c, sizeof(char));
      buf[i] = c;
      ++i;
   } while (c != '\0');
   
   return buf;
}

inline void sw_s_int (int socket, int16_t s) {
   
   s = htons (s);
   sock_write (socket, &s, sizeof (int16_t));
}

inline void sw_int (int socket, int32_t i) {
   
   i = htonl (i);
   sock_write (socket, &i, sizeof (int32_t));
   
}
